import pandas as pd


def mean_encode(df: pd.DataFrame, groupby: str, target: str):
    """
    Groups a Pandas DataFrame via a given column and return
    the mean of the target variable for that grouping.

    Args:
        df (pd.DataFrame): Pandas DataFrame.
        groupby (str): Column to group by.
        target (str): Target variable column.

    Returns:
        pandas.Dataframe: Mean for the target variable across the group.
    """
    mean_encoded = df.groupby(groupby)[target].mean()
    return df[groupby].map(mean_encoded)



from simple_salesforce import Salesforce
import pandas as pd
import requests
from io import StringIO
import os
from ..cleaning import clean_col_names


class SalesforceConn:
    def __init__(self, user, password, token):
        self.sf_instance = 'https://rydoo.lightning.force.com/'
        self.sf = Salesforce(user, password, token)

    def get_report(self, reportid):
        export = '?isdtp=p1&export=1&enc=UTF-8&xf=csv'
        sfUrl = self.sf_instance + reportid + export
        response = requests.post(
            sfUrl,
            headers=self.sf.headers,
            cookies={'sid': self.sf.session_id}
            )
        
        download_report = response.content.decode('utf-8')

        df = pd.read_csv(StringIO(download_report))
        df.columns = clean_col_names(df.columns)
        return df

    def object_names(self):
        objects = self.sf.restful('sobjects')
        return [object['name'] for object in objects['sobjects']]
    
    def query(self, query):
        return self.sf.query_all(query)


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mydstools', 'mydstools.connectors']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.6.3,<4.0.0',
 'pandas>=1.5.3,<2.0.0',
 'seaborn>=0.12.2,<0.13.0',
 'simple-salesforce>=1.12.3,<2.0.0']

setup_kwargs = {
    'name': 'mydstools',
    'version': '0.1.2',
    'description': 'Data Science Toolbox',
    'long_description': '# mydstools\n\n`mydstools`: **Data Science Toolbox**\n\nFollow the project on:\n\n* [Github](https://github.com/antobzzll/mydstools)\n* [PyPI](https://pypi.org/project/mydstools/)\n\n## Installation\n\n```bash\n$ pip install mydstools\n```\n\n## Usage\n\nFor a complete usage guide, please refer to the [example notebook](docs/example.ipynb).\n\n## Contributing\n\nInterested in contributing? Check out the [contributing guidelines](CONTRIBUTING.md). Please note that this project is released with a [Code of Conduct](CONDUCT.md). By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`mydstools` was created by Antonio Buzzelli. It is licensed under the terms of the [MIT license](LICENSE).\n\n## Credits\n\n`mydstools` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Antonio Buzzelli',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns


def distribution_analysis(series: pd.Series, 
                          label: str = None, 
                          binwidth: int = None):
    """
    Function that combines `distribution_stats()` and `distribution_plots()`.

    Args:
        series (pd.Series): Input pandas.Series
        label (str, optional): Label that describes the series. Defaults to None.
        binwidth (int, optional): Width of the histogram bins. Defaults to None.
    """
    if not label:
        label = series.name
    distribution_plots(series, label, binwidth)

    stats = distribution_stats(series)
    for k, v in stats.items():
        print(k, "\t", v)


def distribution_plots(series: pd.Series, label: str = None, binwidth=None):
    """
    Creates a distribution plot for a given series.
    It takes in two parameters: the series and an optional label. 
    The code then creates two subplots with a boxplot and a histogram 
    with kernel density estimation on the right. The binwidth parameter 
    can be used to adjust the bin size of the histogram.

    Args:
        series (pd.Series): Input pandas.Series
        label (str, optional): Label that describes the series. Defaults to None.
        binwidth ([type], optional): Width of the histogram bins. Defaults to None.
    """
    if not label:
        label = series.name
    # distribution plots
    fig, ax = plt.subplots(1, 2, figsize=(15, 5))
    sns.boxplot(ax=ax[0], x=series)
    sns.histplot(ax=ax[1], x=series, binwidth=binwidth, kde=True)
    plt.suptitle(f"Distribution of {label}")
    plt.show()


def distribution_stats(series: pd.Series):
    """
    Returns a dictionary with the most common statistics for 
    a given distribution.

    Args:
        series (pd.Series): [description]

    Returns:
        [type]: [description]
    """
    stats = series.describe()

    iqr = stats['75%'] - stats['25%']
    stats['IQR'] = iqr
    low_threshold = stats['25%'] - (1.5 * iqr)
    if low_threshold >= stats['min']:
        stats['lo-wh'] = low_threshold
    stats['hi-wh'] = stats['75%'] + (1.5 * iqr)
    stats['skew'] = series.skew()

    return stats


def obj_vars_unique(df: pd.DataFrame, verbose=False):
    """
    Creates a dictionary or prints the dataframe's unique values for object
    variables.

    Args:
        df (pd.DataFrame): input DataFrame
        verbose (bool, optional): If True, prints unique values. Defaults to False.

    Returns:
        dict: dictionnary of column names and unique values.
    """
    cat_vars = [var for var in df.columns if df[var].dtype in ['object']]
    cat_vars_unique = {}

    for v in cat_vars:
        uniques = df[v].unique()
        cat_vars_unique[v] = uniques
        if verbose:
            print(f"{v}: {uniques}")

    if not verbose:
        return cat_vars_unique


def plot_corr(df: pd.DataFrame, var: str = None):
    """
    The function calculates the correlation between the variables in the 
    DataFrame and creates a heatmap to visualize it. If the string variable 
    is specified, it will create a heatmap that shows the features correlating 
    with that variable. Otherwise, it will create a heatmap of all variables 
    in the DataFrame.

    Args:
        df (pd.DataFrame): input dataframe
        var (str, optional): name of the variable to focus on. Defaults to None.
    """
    corr = df.corr()
    if var:
        heatmap = sns.heatmap(corr[[var]].sort_values(by=var, ascending=False)[1:],
                              vmin=-1, vmax=1, annot=True, cmap='BrBG')
        heatmap.set_title(f'Features correlating with `{var}`')
    else:
        sns.heatmap(corr, vmin=-1, vmax=1,
                    # mask=mask,
                    annot=True, cmap='BrBG')
    plt.show()


def perc_missing_values(df: pd.DataFrame):
    """
    The function calculates the percentage of missing values in the DataFrame 
    and returns a list of the columns with non-zero missing values, 
    sorted in descending order.

    Args:
        df (pd.DataFrame): input dataframe

    Returns:
        list: list of the columns with non-zero missing values
    """
    missing = df.isnull().sum().sort_values(ascending=False)/df.shape[0]
    return missing[missing != 0]


def check_column(column: pd.Series):
    """
    This code takes in a pandas series as an argument and prints out 
    information about the column. It prints out the data type, the number 
    of null values and non-null values, and if the data type is int or float 
    it will print out summary statistics. If the data type is not int or 
    float it will print out the number of unique values and list them.

    Args:
        column (pd.Series): input series
    """
    n_unique = len(column.unique())
    nulls = column.isnull().sum()
    non_nulls = column.shape[0] - nulls
    print(f'Dtype: {column.dtype}')
    print(f'NULL values: {nulls} ({nulls/column.shape[0]})')
    print(f'NON-NULL values: {non_nulls} ({non_nulls/column.shape[0]})')

    if column.dtype in (int, float):
        print(column.describe())
    else:
        print(f'Unique values: {n_unique} ({n_unique/non_nulls})')
        print(column.unique())



# This module is part of mydstools.
# Please refer to https://github.com/antobzzll/dstoolbox

import numpy as np
import pandas as pd
import re


def clean_col_names(df_columns: pd.Index, wlen: int = 0) -> list:
    """
    Method for cleaning column names of a `pandas.DataFrame`.

    Args:
        df_columns (pd.Index): dataframe's dirty column names.
        wlen (int, optional): length of cleaned words. Defaults to 0.

    Raises:
        ValueError: if `df_columns` is not a `pd.Index`.

    Returns:
        list: list of new column names to be assigned to `df.columns`.
    """

    # usage:
    # df.columns = utilities.clean_col_names(df.columns)

    try:
        df_columns = df_columns.to_list()
    except AttributeError:
        raise ValueError(
            "Invalid argument type. `df_columns` takes a "
            "`pandas.Index` type.") from None
    else:
        def _fix(target: str):
            target = target.lower().split()
            res = []
            for elem in target:
                elem = re.sub(r'[^\w]', '', elem)
                
                if elem:
                    if wlen > 0:
                        res.append(elem[:wlen])
                    else:
                        res.append(elem)
            return '_'.join(res)

        return list(map(_fix, df_columns))


def currency_to_float(
        target: str | list | tuple | pd.Series
        ) -> str | list | tuple | pd.Series:
    """Automatically cleans a currency containing variable and prepares it 
    for analysis by transforming it to `float` type. 
    Target variable of type `str`, `list`, `tuple`, or `pandas.Series`.

    Args:
        target (str | list | tuple | pandas.Series): target variable.

    Returns:
        [str | list | tuple | pandas.Series]: cleaned target variable.
    """
    symbols = ['$', '€', '£', '¥', '₣', '₹', 'د.ك', 'د.إ', '﷼', '₻', '₽',
               '₾', '₺', '₼', '₸', '₴', '₷', '฿', '원', '₫', '₮', '₯',
               '₱', '₳', '₵', '₲', '₪', '₰']

    codes = ['AFN', 'ALL', 'DZD', 'USD', 'EUR', 'AOA', 'XCD', 'ARS', 'AMD',
             'AWG', 'AUD', 'AZN', 'BSD', 'BHD', 'BDT', 'BBD', 'BYN', 'BZD',
             'XOF', 'BMD', 'BTN', 'INR', 'BOB', 'BOV', 'BAM', 'BWP', 'NOK',
             'BRL', 'BND', 'BGN', 'BIF', 'CVE', 'KHR', 'XAF', 'CAD', 'KYD',
             'CLF', 'CLP', 'CNY', 'COP', 'COU', 'KMF', 'CDF', 'NZD', 'CRC',
             'HRK', 'CUC', 'CUP', 'ANG', 'CZK', 'DKK', 'DJF', 'DOP', 'EGP',
             'SVC', 'ERN', 'ETB', 'FKP', 'FJD', 'XPF', 'GMD', 'GEL', 'GHS',
             'GIP', 'GTQ', 'GBP', 'GNF', 'GYD', 'HTG', 'HNL', 'HKD', 'HUF',
             'ISK', 'IDR', 'XDR', 'IRR', 'IQD', 'ILS', 'JMD', 'JPY', 'JOD',
             'KZT', 'KES', 'KPW', 'KRW', 'KWD', 'KGS', 'LAK', 'LBP', 'LSL',
             'ZAR', 'LRD', 'LYD', 'CHF', 'MOP', 'MGA', 'MWK', 'MYR', 'MVR',
             'MRU', 'MUR', 'XUA', 'MXN', 'MXV', 'MDL', 'MNT', 'MAD', 'MZN',
             'MMK', 'NAD', 'NPR', 'NIO', 'NGN', 'OMR', 'PKR', 'PAB', 'PGK',
             'PYG', 'PEN', 'PHP', 'PLN', 'QAR', 'MKD', 'RON', 'RUB', 'RWF',
             'SHP', 'WST', 'STN', 'SAR', 'RSD', 'SCR', 'SLE', 'SGD', 'XSU',
             'SBD', 'SOS', 'SSP', 'LKR', 'SDG', 'SRD', 'SZL', 'SEK', 'CHE',
             'CHW', 'SYP', 'TWD', 'TJS', 'TZS', 'THB', 'TOP', 'TTD', 'TND',
             'TRY', 'TMT', 'UGX', 'UAH', 'AED', 'USN', 'UYI', 'UYU', 'UZS',
             'VUV', 'VEF', 'VED', 'VND', 'YER', 'ZMW', 'ZWL']

    currencies = symbols + codes

    def _fix(string: str):
        
        # value and currency definitions
        if type(string) == float:
            raise ValueError(f"{string} not a string")
        
        string = string.split()
        if len(string) == 1:
            for c in currencies:
                if c in string[0].upper(): # check if symbols
                    currency = c
                    value = string[0].upper().replace(currency, '')
            if not 'value' in locals():
                currency = ''
                value = string[0]
        else:
            if string[1].upper() in currencies:
                currency = string[1].upper()
                value = string[0]
            elif string[0].upper() in currencies:
                currency = string[0].upper()
                value = string[1]
            else:
                currency = None
                value = string[0]

        # commas and dots manipulations
        first_comma_pos = re.search(r'[,]', value)
        first_dot_pos = re.search(r'[.]', value)

        if all(ele is not None for ele in [first_comma_pos, first_dot_pos]):
            # if both commas and dots have been found
            if first_comma_pos.span()[0] < first_dot_pos.span()[0]:
                value = value.replace(',', '')
            else:
                value = value.replace('.', '')
                value = value.replace(',', '.')
            ret_float = float(value)
            
        elif not any(ele is not None for ele in [first_comma_pos,
                                                 first_dot_pos]):
            ret_float = float(value)
            
        else:
            if len(re.findall(r'[.]', value)) == 1:
                ret_float = float(value)
            elif len(re.findall(r'[,]', value)) == 1:
                value = value.replace(',', '.')
                try:
                    ret_float = float(value)
                except ValueError:
                    ret_float = np.nan
            else:
                ret_float = np.nan

        # final return
        return ret_float, currency


    if type(target) is str:
        return _fix(target)
    else:
        values = []
        currs = []

        if type(target) is list:
            for t in target:
                v, c = _fix(t)
                values.append(v)
                currs.append(c)
            return values, currs
       
        elif type(target) is tuple:
            for t in target:
                v, c = _fix(t)
                values.append(v)
                currs.append(c)
            return tuple(values), tuple(currs)

        elif type(target) is pd.Series:
            for _, t in target.items():
                v, c = _fix(t)
                values.append(v)
                currs.append(c)
            return pd.Series(values), pd.Series(currs)